const { expect } = require('chai')
const { Injectable } = require('flitter-di')

const Scope = require('../../src/model/Scope')

describe('the scope base class', function() {
    let scope
    let config
    beforeEach(function() {
        config = {}
        scope = new Scope(config)
    })

    it('should be a functional class', function() {
        expect(Scope).to.be.a('function')
        expect(scope).to.be.an.instanceof(Scope)
        expect(scope).to.be.an.instanceof(Injectable)
    })

    it('should populate the config property', function() {
        expect(scope.config).to.be.equal(config)
        expect((new Scope).config).to.be.an('object')
    })

    describe('the filter method placeholder', function() {
        it('should be a function', function() {
            expect(scope.filter).to.be.a('function')
        })

        it('should expect one argument', function() {
            expect(scope.filter.length).to.be.equal(1)
        })

        it('should return a promise', function() {
            expect(scope.filter({})).to.be.a('promise')
        })

        it('should resolve to the original filter', async function() {
            const a = {}
            expect(await scope.filter(a)).to.be.equal(a)
        })
    })

    describe('the save method placeholder', function() {
        it('should be a function', function() {
            expect(scope.save).to.be.a('function')
        })

        it('should expect one argument', function() {
            expect(scope.save.length).to.be.equal(1)
        })

        it('should return a promise', function() {
            expect(scope.save({})).to.be.a('promise')
        })
    })
})
